/*
** JabberX (Jabber Client)
** Copyright (c) 1999-2001 Gurer Ozen <palpa@jabber.org>
**
** This code is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License.
**
** one-on-one chat
*/

#include "common.h"
#include <math.h>

#define THREAD_LEN 31

textbuf *textbuf_from_id(iksid *id);
char *new_message_thread(void);
void show_chat_msg(textbuf *tb, ikspak *pak);

#define TC_LEN     62
static char* tc = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";

void cmd_chat(char *arg)
{
        textbuf *tb;
        iksid *id;
        juser *u;

        if (!arg || *arg == '\0')
        {
                gui_attr(GA_NOTICE);
                gui_print("Usage: chat <jid or name>\n");
                return;
        }

	u = roster_find_by_name(arg, 1);
	if (!u) u = roster_find_by_name(arg, 0);
	
	if(u)
		id = iks_id_new(NULL, iks_id_printx(u->id, IKS_ID_USER | IKS_ID_SERVER));
	else
                id = iks_id_new(NULL, arg);

        tb = textbuf_from_id(id);

        if (!tb)
        {
                tb = ui_buffer_new();
  	        tb->type = TB_CHAT;
	        tb->id = id;
                tb->thread = new_message_thread();
        }

        ui_buffer_show(tb);
}

void cmd_close(char *arg) 
{
        if(curbuf->type != TB_CHAT) return;

	ui_buffer_delete(curbuf);
}

char *new_message_thread(void)
{
        char *t = malloc(THREAD_LEN + 1);
        int i;

        for (i = 0; i < THREAD_LEN; i++) 
        {
                t[i] = tc[rand() % (TC_LEN + 1)];
        }

        t[THREAD_LEN] = '\0';

        return t;
}

int chat_msg(ikspak *pak)
{
        char *t;
        textbuf *tb;

        t = iks_find_cdata(pak->x, "thread");

        if (t)
        {
                  tb = textbuf_from_id(pak->from);

                  if (tb)
                  {
                        if (tb->type == TB_CHAT) 
                        {
                                  if (!tb->thread)
                                          tb->thread = strdup(t);
                                  else if (strcmp(tb->thread, t)) 
                                  {
                                          free(tb->thread);
                                          tb->thread = strdup(t);
                                  }

                                  show_chat_msg(tb, pak);
                                  if (options.popup_chat)
                                          ui_buffer_show(tb);
                                  return 1;
                        }
                  }
                  else if (options.auto_chat)
                  {
                        tb = ui_buffer_new();
                        tb->type = TB_CHAT;
                        tb->id = iks_id_new(NULL, iks_id_printx(pak->from, IKS_ID_USER | IKS_ID_SERVER));
                        tb->thread = strdup(t);
                        show_chat_msg(tb, pak);
                        if (options.popup_chat)
                                 ui_buffer_show(tb);
                        return 1;
                  }
        }

        return 0;
}

textbuf *textbuf_from_id(iksid *id)
{
        textbuf *tb;

        for(tb = list_first(&textbufs); tb; tb = list_next(tb))
	{
		if(iks_id_cmpx(id, tb->id, IKS_ID_USER | IKS_ID_SERVER) == 0)
                  return tb;  
		
	}

        return NULL;
}

void show_chat_msg(textbuf *tb, ikspak *pak)
{
        juser *u = roster_find(pak->from);
        char * body = iks_find_cdata(pak->x, "body");

        gui_attr(GA_BODY);

	if(u && u->name)
		ui_print(tb, "<%s> %s\n", u->name, body);
	else
		ui_print(tb, "<%s> %s\n", iks_id_printx(pak->from, IKS_ID_USER | IKS_ID_SERVER), body);
}

void chat_send(textbuf *tb, char *msg)
{
	iks *x;

	x = iks_make_msg(IKS_TYPE_CHAT, iks_id_printx(tb->id, IKS_ID_USER | IKS_ID_SERVER), msg, tb->thread);
	iks_send(net.parser, x);
        iks_delete(x);

        ui_print(tb, "<%s> %s\n", net.id->user, msg);        
}
