/*
** JabberX (Jabber Client)
** Copyright (c) 1999-2001 Gurer Ozen <palpa@jabber.org>
**
** This code is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License.
**
** message handling
*/

#include "common.h"


void message_parse(ikspak *pak)
{
	iks *x, *g;
	juser *u;
	char *body = NULL, *errnum;
        textbuf *tb;

	if(pak->subtype == IKS_TYPE_ERROR)
	{
		gui_attr(GA_SYSMSG);
		gui_print("Error");
		x = iks_find(pak->x, "error");
		if(x)
		{
			errnum = iks_find_attrib(x, "code");
			if(errnum)
				gui_print("[%s]: ", errnum);
			else
				gui_print(": "); 

			gui_attr(GA_BODY);
			gui_print("%s\n", iks_cdata(iks_child(x)));
		}
		else
		{
			gui_attr(GA_BODY);
			gui_print(": No error message!\n");
		}
	}

	if(pak->subtype == IKS_TYPE_GROUPCHAT)
	{
		gchat_msg(pak);
		return;
	}

        if (chat_msg(pak))
                return;

         /* FIXME: rectify this with above call. */
   
         tb = target_textbuf(pak);
 
         if (tb)
         {
                 if (tb->type == TB_GCHAT)
                 {
                         gchat_whisper(tb, pak);
                 }
                 else
                 {
                         /* 1-on-1 chat? */
                 }
 
                 return;
         }

	gui_xprint(UIF_COPYMAIN|UIF_SETATTR|UIF_TIMESTAMP,GA_HEADER,"From: ");
	u = roster_find(pak->from);
	if(u && u->name)
	{
		gui_xprint(UIF_COPYMAIN|UIF_SETATTR|UIF_NEWLINE,GA_BODY,
			   "%s (%s)",u->name,iks_id_print(u->id));
		ui_set_tab_content("/s %s ",u->name);
	}
	else
	{
		gui_xprint(UIF_COPYMAIN|UIF_SETATTR|UIF_NEWLINE,GA_BODY,
			   "%s",iks_id_print(pak->from));
		ui_set_tab_content("/s %s ", iks_id_print(pak->from));
	}

	x = iks_find(pak->x, "thread");
	if(x)
	{
		gui_xprint(UIF_COPYMAIN|UIF_SETATTR|UIF_EXTRANL,GA_HEADER,
			   "Thread: ");
		gui_xprint(UIF_COPYMAIN|UIF_SETATTR|UIF_NEWLINE|UIF_STRIPNL|UIF_EXTRANL,GA_BODY,
			   "%s", iks_cdata(iks_child(x)));

	}

	x = iks_find(pak->x, "body");
	if(x)
	{
		gui_xprint(UIF_COPYMAIN|UIF_SETATTR|UIF_EXTRANL,GA_HEADER,
			   "Message: ");
		gui_xprint(UIF_COPYMAIN|UIF_SETATTR|UIF_NEWLINE|UIF_STRIPNL|UIF_EXTRANL,GA_BODY,
			   "%s", body = iks_cdata(iks_child(x)));
	}

        x = iks_find(pak->x, "x");
        if (!iks_strcmp(iks_find_attrib(x, "xmlns"), "jabber:x:roster"))
        {
                gui_attr(GA_HEADER);
                gui_print("Contacts:\n");

                gui_attr(GA_BODY);

                for (x = iks_child(x); x; x = iks_next(x))
                {
                           if(iks_type(x) == IKS_TAG)
		           {
			               if(!iks_strcmp(iks_name(x), "item"))
                                       {
                                                gui_print("-> %s (%s)",
                                                          iks_find_attrib(x, "name"),
                                                          iks_find_attrib(x, "jid"));
                                                ui_set_tab_content("/su %s", iks_find_attrib(x, "jid"));
                                                
                                                for (g = iks_child(x); g; g = iks_next(g))
                                                {
                                                          if (iks_type(g) == IKS_TAG)
                                                          {
                                                                  if (!iks_strcmp(iks_name(g), "group")) {
                                                                            gui_print(" %s", iks_cdata(iks_child(g)));
                                                                            if (iks_next(g))
                                                                                      gui_print(",");
                                                                  }
                                                          }
                                                }

                                                gui_print("\n");
                                       }
                           }
                }
        }
	pl_handle_message(iks_id_print(pak->from), body);

	iks_delete(pak->x);
	on_exec(options.on_message);
}


void cmd_send(char *arg)
{
	iks *x, *y;
	char *to=NULL, *body=NULL;
	juser *u;

	if(arg && *arg != '\0')
	{
		to = strtok(arg, " ");
		if(to) body = strtok(NULL, "");
	}

	if(!to || !body)
	{
		gui_attr(GA_NOTICE);
		gui_print("Usage: send <jid> <msg>\n");
		if (!options.compact_lines)
			gui_print ("\n");
		return;
	}

	to = strdup(to);

	ui_set_tab_content("/s %s ",to);

	x = iks_new("message");

	/* Check for available people first */
	u = roster_find_by_name(to, 1);
	if (!u) u = roster_find_by_name(to, 0);
	
	if(u)
		iks_insert_attrib(x, "to", iks_id_print(u->id));
	else
		iks_insert_attrib(x, "to", to);

	y = iks_insert(x, "body");
	iks_insert_cdata(y, body, -1);


	if( !options.show_sent ) 
	{
		gui_attr(GA_NOTICE);
		gui_print("message sent\n");
		if ( ! options.compact_lines)
			gui_print("\n");
	} else {
		gui_xprint(UIF_SETATTR|UIF_TIMESTAMP,GA_NOTICE,"To: ");
		if(u && u->name)
			gui_xprint(UIF_SETATTR|UIF_STRIPNL|UIF_NEWLINE|UIF_EXTRANL,GA_BODY,
				   "%s (%s)",u->name,iks_id_print(u->id));
		else
			gui_xprint(UIF_SETATTR|UIF_STRIPNL|UIF_NEWLINE|UIF_EXTRANL,GA_BODY,
				   "%s",to);
		gui_xprint(UIF_SETATTR|UIF_EXTRANL,GA_NOTICE,"Message: ");
		gui_xprint(UIF_SETATTR|UIF_NEWLINE|UIF_STRIPNL|UIF_EXTRANL,GA_BODY,"%s",iks_cdata(iks_child(y)));
	}

	iks_send(net.parser, x);
	iks_delete(x);
	free(to);

}

